#!/usr/bin/env python
#
# fsl.get_standard - Generate paths to FSL standard space images.
#
# Author: Paul McCarthy <pauldmccarthy@gmail.com>
#
"""The fsl.get_standard module can be used to locate standard template files,
e.g. $FSLDIR/data/standard/MNI152_T1_2mm.nii.gz.  It is intended to be used by
tools which can be used with standard templates other than the MNI152.
"""


import functools as ft
import os.path   as op
import              os
import              sys
import              argparse

from collections import defaultdict

from typing import Dict, List, Sequence


__version__ = '0.4.0'
"""fsl_get_standard version number."""


class Manifest:
    """Simple container class for storing template file paths. """


    @staticmethod
    def getManifest() -> 'Manifest':
        """Loads and returns a template manifest, based on the $FSL_STANDARD
        environment variable.
        """

        fsldir = os.environ['FSLDIR']
        stddir = os.environ.get('FSL_STANDARD')

        if stddir is None:
            return Manifest.defaultManifest()

        # FSL_STANDARD may be an identifier for a
        # dataset in ${FSLDIR}/data/${FSL_STANDARD}/
        if not op.exists(stddir):
            stddir = op.join(fsldir, 'data', stddir)

        # Or may be set to a directory containing
        # a standard template data set.
        else:
            stddir = op.abspath(stddir)

        manifestFile = op.join(stddir, 'manifest.txt')

        if not op.exists(manifestFile):
            raise Exception(f'Manifest file missing: {manifestFile}')

        return Manifest.loadManifestFile(manifestFile)


    @staticmethod
    def defaultManifest() -> 'Manifest':
        """Called by Manifest.getManifest when the FSL_STANDARD variable is
        not set.

        Returns the manifest for the default FSL MNI152 standard template
        images, contained in $FSLDIR/data/standard/.
        """

        fsldir    = os.environ['FSLDIR']
        stddir    = op.join(fsldir, 'data', 'standard')
        manifest  = Manifest()
        filepaths = [
            ('whole_head', 'T1', 2,   op.join(stddir, 'MNI152_T1_2mm.nii.gz')),
            ('whole_head', 'T1', 1,   op.join(stddir, 'MNI152_T1_1mm.nii.gz')),
            ('whole_head', 'T1', 0.5, op.join(stddir, 'MNI152_T1_0.5mm.nii.gz')),
            ('brain',      'T1', 2,   op.join(stddir, 'MNI152_T1_2mm_brain.nii.gz')),
            ('brain',      'T1', 1,   op.join(stddir, 'MNI152_T1_1mm_brain.nii.gz')),
            ('brain_mask', 'T1', 2,   op.join(stddir, 'MNI152_T1_2mm_brain_mask.nii.gz')),
            ('brain_mask', 'T1', 1,   op.join(stddir, 'MNI152_T1_1mm_brain_mask.nii.gz')),
        ]

        for imgtype, modality, resolution, filepath in filepaths:
            manifest.add(imgtype, modality, resolution, filepath)

        return manifest


    @staticmethod
    def loadManifestFile(manifestFile : str) -> 'Manifest':
        """Called by Manifest.getManifest. Loads and returns a manifest from a
        manifest file.
        """

        manifest = defaultdict(dict)
        dirname  = op.dirname(op.abspath(manifestFile))

        with open(manifestFile, 'rt') as f:
            lines = f.readlines()

        lines = [l.strip() for l in lines]
        lines = [l         for l in lines if not (l == '' or l.startswith('#'))]

        manifest = Manifest()

        for line in lines:

            try:
                words = line.split(',')

                # <= 0.2.0 - type, resolution, filepath
                if len(words) == 3:
                    imgtype    =       words[0].strip()
                    resolution = float(words[1].strip())
                    modality   =       None
                    fpath      =       words[2].strip()
                # >= 0.3.0 - type, resolution, modality, filepath
                elif len(words) == 4:
                    imgtype    =       words[0].strip()
                    resolution = float(words[1].strip())
                    modality   =       words[2].strip()
                    fpath      =       words[3].strip()
                else:
                    raise ValueError('Invalid line in manifest '
                                     f'{manifestFile}: {line}')

                fpath = op.join(dirname, fpath)

                if not op.exists(fpath):
                    raise ValueError(f'Cannot find file: {fpath}')

            except Exception as e:
                raise Exception(line) from e

            manifest.add(imgtype, modality, resolution, fpath)

        return manifest


    def __init__(self):
        """Don't create a Manifest object - use the Manifest.getManifest method
        instead.
        """

        self.__manifest    = {}
        self.__modalities  = []
        self.__imageTypes  = []
        self.__resolutions = []


    def add(self,
            imageType  : str,
            modality   : str,
            resolution : float,
            filepath   : str):
        """Add a template file to the manifest. """

        man   = self.__manifest
        itype = imageType
        mod   = modality
        res   = resolution

        if not op.exists(filepath):
            raise Exception('Template file does not exist: {filepath}')

        if itype not in man:             man[itype]           = {}
        if mod   not in man[itype]:      man[itype][mod]      = {}
        if res   not in man[itype][mod]: man[itype][mod][res] = {}

        man[itype][mod][res] = filepath

        if itype not in self.__imageTypes:  self.__imageTypes .append(itype)
        if mod   not in self.__modalities:  self.__modalities .append(mod)
        if res   not in self.__resolutions: self.__resolutions.append(res)


    def get(self,
            imageType  : str   = None,
            modality   : str   = None,
            resolution : float = None) -> str:
        """Retrieve a template file path with the specified properties. """

        if imageType  is None: imageType  = self.__imageTypes[ 0]
        if modality   is None: modality   = self.__modalities[ 0]
        if resolution is None: resolution = self.__resolutions[0]

        if imageType not in self.__imageTypes:
            raise Exception(f'Unknown image type: {imageType}')
        if modality not in self.__modalities:
            raise Exception(f'Unknown modality: {modality}')
        if resolution not in self.__resolutions:
            raise Exception(f'Unknown resolution: {resolution}')

        try:
            return self.__manifest[imageType][modality][resolution]
        except KeyError:
            raise Exception('No template for requested image type/modality/'
                            f'resolution: {imageType}/{modality}/{resolution}')

    @property
    def imageTypes(self) -> List[str]:
        """Return a list of all image types in the manifest. """
        return list(self.__imageTypes)


    @property
    def modalities(self) -> List[str]:
        """Return a list of all modalities in the manifest. """
        return list(self.__modalities)


    @property
    def resolutions(self) -> List[float]:
        """Return a list of all resolutions in the manifest. """
        return list(self.__resolutions)
